@@ -32,7 +32,7 @@ from gi.repository import Gtk, GdkPixbuf, GLib, Gio  # pylint: disable=E0611
 import logging
 logger = logging.getLogger('mugshot')
 
-from mugshot_lib import Window, SudoDialog, helpers
+from mugshot_lib import Window, SudoDialog, AccountsServiceAdapter, helpers
 
 try:
     from mugshot.CameraMugshotDialog import CameraMugshotDialog
@@ -198,9 +198,6 @@ class MugshotWindow(Window):
         self.email_entry = builder.get_object('email')
         self.fax_entry = builder.get_object('fax')
 
-        # Users without sudo rights cannot change their name.
-        self.set_name_editable(SudoDialog.check_dependencies(['chfn']))
-
         # Stock photo browser
         self.stock_browser = builder.get_object('stock_browser')
         self.iconview = builder.get_object('stock_iconview')
@@ -219,6 +216,14 @@ class MugshotWindow(Window):
 
         self.tmpfile = None
 
+        self.accounts_service = \
+            AccountsServiceAdapter.MugshotAccountsServiceAdapter(username)
+
+        # Users without sudo rights cannot change their name.
+        if not self.accounts_service.available():
+            self.set_name_editable(SudoDialog.check_dependencies(['chfn']))
+        self.set_phone_editable(SudoDialog.check_dependencies(['chfn']))
+
         # Populate all of the widgets.
         self.init_user_details()
 
@@ -228,22 +233,26 @@ class MugshotWindow(Window):
         self.last_name_entry.set_sensitive(editable)
         self.initials_entry.set_sensitive(editable)
 
+    def set_phone_editable(self, editable):
+        self.home_phone_entry.set_sensitive(editable)
+        self.office_phone_entry.set_sensitive(editable)
+
     def init_user_details(self):
         """Initialize the user details entries and variables."""
         # Check for .face and set profile image.
         logger.debug('Checking for ~/.face profile image')
         face = os.path.join(home, '.face')
-        logger.debug('Checking AccountsService for profile image')
-        image = self.accounts_service_get_user_image()
 
-        # AccountsService may not be supported or desired.
-        if image is None:
+        logger.debug('Checking AccountsService for profile image')
+        if not self.accounts_service.available():
+            # AccountsService may not be supported or desired.
             logger.debug("AccountsService is not supported.")
             self.updated_image = face
             self.set_user_image(face)
 
         # If it is supported, process and compare to ~/.face
         else:
+            image = self.accounts_service.get_icon_file()
             logger.debug('Found profile image: %s' % str(image))
 
             if os.path.isfile(face):
@@ -267,8 +276,8 @@ class MugshotWindow(Window):
         self.initials = user_details['initials']
         self.home_phone = user_details['home_phone']
         self.office_phone = user_details['office_phone']
-        email = user_details['email']
-        fax = user_details['fax']
+        self.email = user_details['email']
+        self.fax = user_details['fax']
 
         # Populate the GtkEntries.
         logger.debug('Populating entries')
@@ -277,25 +286,29 @@ class MugshotWindow(Window):
         self.initials_entry.set_text(self.initials)
         self.office_phone_entry.set_text(self.office_phone)
         self.home_phone_entry.set_text(self.home_phone)
-        self.email_entry.set_text(email)
-        self.fax_entry.set_text(fax)
+        self.email_entry.set_text(self.email)
+        self.fax_entry.set_text(self.fax)
 
     # = Mugshot Window ====================================================== #
     def set_user_image(self, filename=None):
         """Scale and set the user profile image."""
         logger.debug("Setting user profile image to %s" % str(filename))
-        if filename:
-            pixbuf = GdkPixbuf.Pixbuf.new_from_file(filename)
-            scaled = pixbuf.scale_simple(128, 128, GdkPixbuf.InterpType.HYPER)
-            self.user_image.set_from_pixbuf(scaled)
-            # Show "Remove" menu item.
-            self.menuitem1.set_visible(True)
-            self.image_remove.set_visible(True)
-        else:
-            self.user_image.set_from_icon_name('avatar-default', 128)
-            # Hide "Remove" menu item.
-            self.menuitem1.set_visible(False)
-            self.image_remove.set_visible(False)
+        if filename and os.path.exists(filename):
+            try:
+                pixbuf = GdkPixbuf.Pixbuf.new_from_file(filename)
+                scaled = pixbuf.scale_simple(128, 128, GdkPixbuf.InterpType.HYPER)
+                self.user_image.set_from_pixbuf(scaled)
+                # Show "Remove" menu item.
+                self.menuitem1.set_visible(True)
+                self.image_remove.set_visible(True)
+                return
+            except GLib.Error:
+                pass
+
+        self.user_image.set_from_icon_name('avatar-default', 128)
+        # Hide "Remove" menu item.
+        self.menuitem1.set_visible(False)
+        self.image_remove.set_visible(False)
 
     def suggest_initials(self, first_name, last_name):
         """Generate initials from first and last name."""
@@ -343,6 +356,9 @@ class MugshotWindow(Window):
                 dialog.destroy()
                 return
 
+        if self.get_as_details_updated():
+            self.save_as_details()
+
         if self.get_libreoffice_details_updated():
             self.set_libreoffice_data()
 
@@ -412,8 +428,9 @@ class MugshotWindow(Window):
                 shutil.copyfile(self.updated_image, face)
 
         # Update AccountsService profile image
-        logger.debug('Photo updated, saving AccountsService profile image.')
-        self.accounts_service_set_user_image(self.updated_image)
+        if self.accounts_service.available():
+            logger.debug('Photo updated, saving AccountsService profile image.')
+            self.accounts_service.set_icon_file(self.updated_image)
 
         # Update Pidgin buddy icon
         self.set_pidgin_buddyicon(self.updated_image)
@@ -421,64 +438,6 @@ class MugshotWindow(Window):
         self.updated_image = None
         return True
 
-    def accounts_service_get_user_image(self):
-        """Get user profile image using AccountsService."""
-        try:
-            bus = Gio.bus_get_sync(Gio.BusType.SYSTEM, None)
-            result = bus.call_sync('org.freedesktop.Accounts',
-                                   '/org/freedesktop/Accounts',
-                                   'org.freedesktop.Accounts',
-                                   'FindUserByName',
-                                   GLib.Variant('(s)', (username,)),
-                                   GLib.VariantType.new('(o)'),
-                                   Gio.DBusCallFlags.NONE,
-                                   -1,
-                                   None)
-            (path,) = result.unpack()
-
-            variant = GLib.Variant('(s)',
-                                   ('org.freedesktop.Accounts.User',))
-            result = bus.call_sync('org.freedesktop.Accounts',
-                                   path,
-                                   'org.freedesktop.DBus.Properties',
-                                   'GetAll',
-                                   variant,
-                                   GLib.VariantType.new('(a{sv})'),
-                                   Gio.DBusCallFlags.NONE,
-                                   -1,
-                                   None)
-            (props,) = result.unpack()
-            return props['IconFile']
-        except GLib.GError:
-            return None
-
-    def accounts_service_set_user_image(self, filename):
-        """Set user profile image using AccountsService."""
-        try:
-            bus = Gio.bus_get_sync(Gio.BusType.SYSTEM, None)
-            result = bus.call_sync('org.freedesktop.Accounts',
-                                   '/org/freedesktop/Accounts',
-                                   'org.freedesktop.Accounts',
-                                   'FindUserByName',
-                                   GLib.Variant('(s)', (username,)),
-                                   GLib.VariantType.new('(o)'),
-                                   Gio.DBusCallFlags.NONE,
-                                   -1,
-                                   None)
-            (path,) = result.unpack()
-
-            bus.call_sync('org.freedesktop.Accounts',
-                          path,
-                          'org.freedesktop.Accounts.User',
-                          'SetIconFile',
-                          GLib.Variant('(s)', (filename,)),
-                          GLib.VariantType.new('()'),
-                          Gio.DBusCallFlags.NONE,
-                          -1,
-                          None)
-        except GLib.GError:
-            pass
-
     def set_pidgin_buddyicon(self, filename=None):
         """Sets the pidgin buddyicon to filename (usually ~/.face).
 
@@ -535,14 +494,37 @@ class MugshotWindow(Window):
             write_prefs.close()
 
     # = chfn functions ============================================ #
+    def get_as_details_updated(self):
+        if not self.accounts_service.available():
+            return False
+        if self.first_name != self.first_name_entry.get_text().strip():
+            return True
+        if self.last_name != self.last_name_entry.get_text().strip():
+            return True
+        if self.email != self.email_entry.get_text().strip():
+            return True
+
+    def save_as_details(self):
+        if not self.accounts_service.available():
+            return True
+        first_name = get_entry_value(self.first_name_entry)
+        last_name = get_entry_value(self.last_name_entry)
+        full_name = "%s %s" % (first_name, last_name)
+        full_name = full_name.strip()
+        email = get_entry_value(self.email_entry)
+        self.accounts_service.set_real_name(full_name)
+        self.accounts_service.set_email(email)
+        return True
+
     def get_chfn_details_updated(self):
         """Return True if chfn-related details have been modified."""
         logger.debug('Checking if chfn details have been modified.')
         modified = False
-        if self.first_name != self.first_name_entry.get_text().strip():
-            modified = True
-        if self.last_name != self.last_name_entry.get_text().strip():
-            modified = True
+        if not self.accounts_service.available():
+            if self.first_name != self.first_name_entry.get_text().strip():
+                modified = True
+            if self.last_name != self.last_name_entry.get_text().strip():
+                modified = True
         if self.home_phone != self.home_phone_entry.get_text().strip():
             modified = True
         if self.office_phone != self.office_phone_entry.get_text().strip():
@@ -608,14 +590,16 @@ class MugshotWindow(Window):
             home_phone = 'none'
 
         # Full name can only be modified by root.  Try using sudo to modify.
-        if SudoDialog.check_dependencies(['chfn']):
-            logger.debug('Updating Full Name...')
-            command = "%s %s -f \"%s\" %s" % (sudo, chfn, full_name, username)
-            if self.process_terminal_password(command, password):
-                self.first_name = first_name
-                self.last_name = last_name
-            else:
-                success = False
+        if not self.accounts_service.available():
+            if SudoDialog.check_dependencies(['chfn']):
+                logger.debug('Updating Full Name...')
+                command = "%s %s -f \"%s\" %s" % (sudo, chfn, full_name,
+                                                  username)
+                if self.process_terminal_password(command, password):
+                    self.first_name = first_name
+                    self.last_name = last_name
+                else:
+                    success = False
 
         logger.debug('Updating Home Phone...')
         command = "%s -h \"%s\" %s" % (chfn, home_phone, username)
@@ -666,29 +650,73 @@ class MugshotWindow(Window):
         # Start with LibreOffice, as users may have configured that first.
         data = self.get_libreoffice_data()
 
-        # Next is passwd, where we override name values with system values.
+        # Prefer AccountsService, GLib, then passwd
+        as_data = self.get_accounts_service_data()
+        gl_data = self.get_glib_data()
         pwd_data = self.get_passwd_data()
-        data['first_name'] = pwd_data['first_name']
-        data['last_name'] = pwd_data['last_name']
-        data['initials'] = pwd_data['initials']
-        if len(data['home_phone']) == 0:
-            data['home_phone'] = pwd_data['home_phone']
-        if len(data['office_phone']) == 0:
-            data['office_phone'] = pwd_data['office_phone']
+
+        for dataset in [as_data, gl_data, pwd_data]:
+            if dataset is None:
+                continue
+            if len(dataset['first_name']) > 0:
+                data['first_name'] = dataset['first_name']
+                data['last_name'] = dataset['last_name']
+                data['initials'] = dataset['initials']
+                break
+
+        for dataset in [as_data, gl_data, pwd_data]:
+            if dataset is None:
+                continue
+            for key in ['home_phone', 'office_phone', 'email', 'fax']:
+                if len(data[key]) == 0:
+                    data[key] = dataset[key]
 
         # Then get data from dconf
-        initials = self.settings['initials']
-        if len(initials) > 0:
-            data['initials'] = initials
-        email = self.settings['email']
+        if len(self.settings['initials']) > len(data['initials']):
+            data['initials'] = self.settings['initials']
         if len(data['email']) == 0:
-            data['email'] = email
+            data['email'] = self.settings['email']
         if len(data['fax']) == 0:
             data['fax'] = self.settings['fax']
 
         # Return all of the finalized information.
         return data
 
+    def split_name(self, name):
+        parts = name.split()
+        initials = ""
+        first = ""
+        last = ""
+        if len(' '.join(parts)) > 0:
+            first = parts[0]
+            if len(parts) > 1:
+                last = ' '.join(parts[1:])
+            for part in parts:
+                if len(part) > 0:
+                    initials += part[0]
+        return {
+            "first": first, "last": last, "initials": initials
+        }
+
+    def get_accounts_service_data(self):
+        if not self.accounts_service.available():
+            return None
+
+        name = self.accounts_service.get_real_name()
+        name = self.split_name(name)
+        email = self.accounts_service.get_email()
+        data = {'first_name': name['first'], 'last_name': name['last'],
+                'home_phone': '', 'office_phone': '',
+                'initials': name['initials'], 'email': email, 'fax': ''}
+        return data
+
+    def get_glib_data(self):
+        name = GLib.get_real_name()
+        name = self.split_name(name)
+        data = {'first_name': name['first'], 'last_name': name['last'],
+                'home_phone': '', 'office_phone': '',
+                'initials': name['initials'], 'email': '', 'fax': ''}
+
     def get_passwd_data(self):
         """Get user details from passwd"""
         # Use getent for current user's details.
@@ -698,31 +726,22 @@ class MugshotWindow(Window):
                 line = line.decode('utf-8')
             line = line.strip()
             logger.debug('Found details: %s' % line.strip())
-            details = line.split(':')[4].split(',', 3)
+            details = line.split(':')[4].split(',')
 
             while len(details) < 4:
                 details.append("")
 
             # Extract the user details
-            name, office, office_phone, home_phone = details
+            name, office, office_phone, home_phone = details[:4]
         except subprocess.CalledProcessError:
             logger.warning("User %s not found in /etc/passwd. "
                            "Mugshot may not function correctly." % username)
             office = ""
             office_phone = ""
             home_phone = ""
+            name = ""
 
-        # Use GLib to get the actual name.
-        name = GLib.get_real_name()
-
-        # Expand the user's fullname into first, last, and initials.
-        initials = name[0]
-        try:
-            first_name, last_name = name.split(' ', 1)
-            initials += last_name[0]
-        except:
-            first_name = name
-            last_name = ''
+        name = self.split_name(name)
 
         # If the variables are defined as 'none', use blank for cleanliness.
         if home_phone == 'none':
@@ -731,9 +750,9 @@ class MugshotWindow(Window):
             office_phone = ''
 
         # Pack the data
-        data = {'first_name': first_name, 'last_name': last_name,
+        data = {'first_name': name['first'], 'last_name': name['last'],
                 'home_phone': home_phone, 'office_phone': office_phone,
-                'initials': initials, 'email': '', 'fax': ''}
+                'initials': name['initials'], 'email': '', 'fax': ''}
 
         return data
 
@@ -747,6 +766,15 @@ class MugshotWindow(Window):
                 'home_phone': '', 'office_phone': '', 'fax': ''}
         if os.path.isfile(prefs_file):
             logger.debug('Getting settings from %s' % prefs_file)
+
+            # Check for file access
+            try:
+                prefs = open(prefs_file, 'r')
+                prefs.close()
+            except PermissionError:
+                logger.debug('Reject: Cannot open file.')
+                return data
+
             for line in open(prefs_file):
                 if "UserProfile/Data" in line:
                     try:
@@ -807,6 +835,15 @@ class MugshotWindow(Window):
                 fax = get_entry_value(self.fax_entry)
                 fax_updated = False
                 tmp_buffer = []
+
+                # Check for file access
+                try:
+                    prefs = open(prefs_file, 'a')
+                    prefs.close()
+                except PermissionError:
+                    logger.debug('Reject: Not updating.')
+                    return
+
                 for line in open(prefs_file):
                     new = None
                     if "UserProfile/Data" in line:
